###############   PSQF 6271 Longitudinal MLM Example 5 using R   #############

# Set width of output and  number of digits to be printed
options("width"=120, "digits"=8) 

# Use this to turn off scientific notation: 
# "fixed notation will be preferred unless it is more than 'scipen' digits wide"
options(scipen=9)

# turn off stars for different alpha levels
options(show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("haven")) install.packages("haven")
library(haven) # To import SAS data with labels as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("nlme")) install.packages("nlme")
library(nlme) # To estimate MLMs with link functions and R cov structures

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do linear combinations in gls

if (!require("emmeans")) install.packages("emmeans")
library(emmeans) # To get model-implied means

if (!require("lmerTest")) install.packages("lmerTest")
library(lmerTest) # To get Satterthwaite DDF in lmer

if (!require("performance")) install.packages("performance")
library(performance) # To get ICC in lmer

if (!require("margins")) install.packages("margins")
library(margins) # To get predicted values like Stata does

if (!require("lavaan")) install.packages("lavaan")
library(lavaan) # To fit SEMs


################################################################################
#####               BEGIN EXAMPLE 5 DATA MANIUPLATION                      #####
################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/22_PSQF6271/PSQF6271_Example5/"
filename = "SAS_Chapter5.sas7bdat"
setwd(dir=filesave)

# Import chapter 5 stacked data with labels
Example5 = read_sas(data_file=paste0(filesave,filename)) 
# Convert to data frame as data frame without labels to use for analysis
Example5 = as.data.frame(Example5)

# Sort data by PersonID (needed for correct RCOV matrix)
Example5 = sort_asc(Example5,PersonID,wave)

# Center time at first occasion
Example5$time=Example5$wave-1
# Make new variable for wave with reference=4 to match other programs
Example5$wave4=relevel(factor(Example5$wave), ref=4)

# Open output text file
txtStart(file=paste0(filesave,"R_Example5_Output.txt"))


################################################################################
#####              BEGIN EXAMPLE 5 MULTILEVEL MODELS                       #####
################################################################################

print("R: Get Variance of Time Predictor for Slope Reliability (=1.263)")
describe(x=Example5$time)  

txtComment(" ") # insert blank space in output


print("R GLS Ch 5: Saturated Means, Unstructured Variance Model")
print("ANSWER KEY for both sides of the model")
SatUN = gls(data=Example5, method="REML", model=outcome~1+factor(wave4), 
            correlation=corSymm(form=~as.numeric(wave4)|PersonID), # Unstructured correlations
            weights=varIdent(form=~1|wave4))                       # Heterogeneous variances
print("Show results with -2LL, incorrect DDF, and total leftover variance")
print("Total variance per occasion is created using SD multiplier")
summary(SatUN); -2*logLik(SatUN); summary(SatUN)$sigma^2 

print("Show R and RCORR matrices for first person") 
R=getVarCov(SatUN, individual="1", type="marginal"); R
RCORR=corMatrix(SatUN$modelStruct$corStruct)[[4]]; RCORR 

print("Wave means and pairwise mean differences with Satterthwaite DDF")
lsmeans(SatUN, "wave4", mode="satterthwaite")
emmeans(ref_grid(SatUN), pairwise~wave4, adjust="none")
print("F-test p-value based on residual DDF instead"); anova(SatUN)

txtComment(" ") # insert blank space in output


print("R LME Ch 5: Saturated Means, Random Intercept Model")
print("ANSWER KEY for means side only")
SatRI = lme(data=Example5, method="REML", outcome~1+factor(wave4), random=~1|PersonID)
print("Show results with -2LL and incorrect DDF") 
summary(SatRI); -2*logLik(SatRI)

print("Show G, R, and V matrices for first person") 
G=getVarCov(SatRI, individual="1", type="random.effects"); G
R=getVarCov(SatRI, individual="1", type="conditional"); R
V=getVarCov(SatRI, individual="1", type="marginal"); V
ICC=(V[[1]][2,1])/(V[[1]][1,1]); print("Show Conditional ICC"); ICC
print("Wave means and pairwise mean differences, and omnibus F-test")
emmeans(ref_grid(SatRI), pairwise~wave4, adjust="none"); anova(SatRI) 

txtComment(" ") # insert blank space in output

print("R LMER Ch 5: Saturated Means, Random Intercept Model")
print("ANSWER KEY for means side only")
SatRIr = lmer(data=Example5, REML=TRUE, formula=outcome~1+factor(wave4)+(1|PersonID))
print("Show results with -2LL using Satterthwaite DDF") 
summary(SatRIr, ddf="Satterthwaite"); llikAIC(SatRIr, chkREML=FALSE)
print("Show Conditional ICC"); icc(SatRIr)
print("Wave means, pairwise mean differences, and omnibus F-test")
emmeans(ref_grid(SatRIr), pairwise~wave4, adjust="none"); anova(SatRIr)  

txtComment(" ") # insert blank space in output


print("R LME Eq 5.1: Empty Means, Random Intercept Model")
EmptyRI = lme(data=Example5, method="REML", outcome~1, random=~1|PersonID)
print("Show results with -2LL and incorrect DDF"); summary(EmptyRI); -2*logLik(EmptyRI) 
print("Show G, R, and V matrices for first person") 
G=getVarCov(EmptyRI, individual="1", type="random.effects"); G
R=getVarCov(EmptyRI, individual="1", type="conditional"); R
V=getVarCov(EmptyRI, individual="1", type="marginal"); V
ICC=(V[[1]][2,1])/(V[[1]][1,1]); print("Show Unconditional ICC"); ICC

txtComment(" ") # insert blank space in output

print("R LMER Eq 5.1: Empty Means, Random Intercept Model")
EmptyRIr = lmer(data=Example5, REML=TRUE, formula=outcome~1+(1|PersonID))
print("Show results using Satterthwaite DDF including -2LL") 
summary(EmptyRIr, ddf="Satterthwaite"); llikAIC(EmptyRIr, chkREML=FALSE)
print("Show Unconditional ICC"); icc(EmptyRIr)
print("Does random intercept improve model fit?")
ranova(EmptyRIr, reduce.term=TRUE) # LRT for removing random intercept

txtComment(" ") # insert blank space in output


print("R LME Eq 5.3: Fixed Linear Time, Random Intercept Model")
FixLinRI = lme(data=Example5, method="REML", outcome~1+time, random=~1|PersonID)
print("Show results using incorrect DDF"); summary(FixLinRI); -2*logLik(FixLinRI)    
print("Show G, R, and V matrices for first person") 
G=getVarCov(FixLinRI, individual="1", type="random.effects"); G
R=getVarCov(FixLinRI, individual="1", type="conditional"); R
V=getVarCov(FixLinRI, individual="1", type="marginal"); V

txtComment(" ") # insert blank space in output

print("R LMER Eq 5.3: Fixed Linear Time, Random Intercept Model")
FixLinRIr = lmer(data=Example5, REML=TRUE, formula=outcome~1+time+(1|PersonID))
print("Show results with -2LL using Satterthwaite DDF") 
summary(FixLinRIr, ddf="Satterthwaite"); llikAIC(FixLinRIr, chkREML=FALSE)
print("Get conditional mean per occasion from value of time predictor")
print("Pred Mean at Time=0"); contest1D(FixLinRIr, ddf="Satterthwaite", L=c(1,0))
print("Pred Mean at Time=1"); contest1D(FixLinRIr, ddf="Satterthwaite", L=c(1,1))
print("Pred Mean at Time=2"); contest1D(FixLinRIr, ddf="Satterthwaite", L=c(1,2))
print("Pred Mean at Time=3"); contest1D(FixLinRIr, ddf="Satterthwaite", L=c(1,3))
print("Predicted outcome means using margins package (but SEs don't work)"); 
prediction(model=FixLinRIr, calculate_se=TRUE, at=list(time=0:3))

txtComment(" ") # insert blank space in output


print("R LME Eq 5.5: Random Linear Time Model")
RandLin = lme(data=Example5, method="REML", outcome~1+time, random=~1+time|PersonID)
print("Show results with -2LL using incorrect DDF"); summary(RandLin); -2*logLik(RandLin)    
print("Show G, R, and V matrices for first person") 
G=getVarCov(RandLin, individual="1", type="random.effects"); G
R=getVarCov(RandLin, individual="1", type="conditional"); R
V=getVarCov(RandLin, individual="1", type="marginal"); V

txtComment(" ") # insert blank space in output

print("R LMER Eq 5.5: Random Linear Time Model")
RandLinr = lmer(data=Example5, REML=TRUE, formula=outcome~1+time+(1+time|PersonID))
print("Show results with -2LL using Satterthwaite DDF") 
summary(RandLinr, ddf="Satterthwaite"); llikAIC(RandLinr, chkREML=FALSE)
print("Pred Mean at Time 0"); contest1D(RandLinr, ddf="Satterthwaite", L=c(1,0))
print("Pred Mean at Time=1"); contest1D(RandLinr, ddf="Satterthwaite", L=c(1,1))
print("Pred Mean at Time=2"); contest1D(RandLinr, ddf="Satterthwaite", L=c(1,2))
print("Pred Mean at Time=3"); contest1D(RandLinr, ddf="Satterthwaite", L=c(1,3))
print("Predicted outcome means using margins package (but SEs don't work)"); 
prediction(model=RandLinr, calculate_se=TRUE, at=list(time=0:3))
print("Does random linear time slope improve fit?")
ranova(RandLinr, reduce.term=TRUE) # Remove random slope and covariance

txtComment(" ") # insert blank space in output


print("R Ch 5: LME Random Linear Time Model + AR1 R Matrix")
RandLinAR1 = lme(data=Example5, method="REML", outcome~1+time, random=~1+time|PersonID,
                 correlation=(corAR1(form=~as.numeric(time)|PersonID)))
print("Show results using incorrect DDF"); summary(RandLinAR1)  
print("Show G, R, and V matrices for first person") 
G=getVarCov(RandLinAR1, individual="1", type="random.effects"); G
R=getVarCov(RandLinAR1, individual="1", type="conditional"); R
V=getVarCov(RandLinAR1, individual="1", type="marginal"); V
print("Does AR1 residual correlation improve fit?")
anova(RandLinAR1, RandLin) # anova compares using LME versions

txtComment(" ") # insert blank space in output


# Toeplitz is not a pre-defined structure in R LME


################################################################################
#####         BEGIN EXAMPLE 5 STRUCTURAL EQUATION MODELS                   #####
################################################################################

# R Syntax using lavaan function: longer but more transparent version of model

# Un-Stack from long to wide format (becomes one row per person)
Example5_wide = reshape(Example5, direction="wide",
                        v.names="outcome", idvar="PersonID", timevar="time")

print("R lavaan Ch 5: Saturated Means, Unstructured Variance Model")
print("ANSWER KEY for both sides of the model")
SyntaxSatUN = "
# Occasion means all estimated
  outcome.0 ~ 1; outcome.1 ~ 1; outcome.2  ~ 1; outcome.3 ~ 1
# Occasion variances all estimated
  outcome.0 ~~ outcome.0; outcome.1 ~~ outcome.1; outcome.2 ~~ outcome.2; outcome.3 ~~ outcome.3
# Occasion covariances all estimated
  outcome.0 ~~ outcome.1; outcome.0 ~~ outcome.2; outcome.0 ~~ outcome.3 
  outcome.1 ~~ outcome.2; outcome.1 ~~ outcome.3; outcome.2 ~~ outcome.3 
"
ModelSatUN = lavaan(model=SyntaxSatUN, data=Example5_wide, estimator="ML", mimic="mplus")
summary(ModelSatUN, fit.measures=TRUE, standardized=FALSE)
print("Model-implied marginal means, variances, and covariances; add correlations")
fitted(object=ModelSatUN); lavInspect(object=ModelSatUN, "cor.ov")

txtComment(" ") # insert blank space in output


print("R lavaan Ch 5: Saturated Means, Random Intercept Model")
print("ANSWER KEY for means side only")
SyntaxSatRI = "
# Define intercept factor by fixing all factor loadings to 1
  FactInt =~ 1*outcome.0 + 1*outcome.1 + 1*outcome.2 + 1*outcome.3
# Fix factor mean to 0 to estimate per-occasion intercepts instead
  FactInt ~ 0
# Estimate intercept factor variance = random intercept variance
  FactInt ~~ (IntVar)*FactInt
# Occasion intercepts all estimated
  outcome.0 ~ 1; outcome.1 ~ 1; outcome.2  ~ 1; outcome.3 ~ 1
# Occasion residual variances all constrained equal
  outcome.0 ~~ (ResVar)*outcome.0; outcome.1 ~~ (ResVar)*outcome.1 
  outcome.2 ~~ (ResVar)*outcome.2; outcome.3 ~~ (ResVar)*outcome.3
"
ModelSatRI = lavaan(model=SyntaxSatRI, data=Example5_wide, estimator="ML", mimic="mplus")
summary(ModelSatRI, fit.measures=TRUE, standardized=FALSE)
print("Model-implied marginal means, variances, and covariances; add correlations")
fitted(object=ModelSatRI); lavInspect(object=ModelSatRI, "cor.ov")

txtComment(" ") # insert blank space in output


print("R lavaan Eq 5.1: Empty Means, Random Intercept Model")
SyntaxEmptyRI = "
# Define intercept factor by fixing all factor loadings to 1
  FactInt =~ 1*outcome.0 + 1*outcome.1 + 1*outcome.2 + 1*outcome.3
# Estimate intercept factor mean = fixed intercept
  FactInt ~ (FixInt)*1
# Estimate intercept factor variance = random intercept variance
  FactInt ~~ (IntVar)*FactInt
# Occasion intercepts all fixed to 0
  outcome.0 ~ 0; outcome.1 ~ 0; outcome.2  ~ 0; outcome.3 ~ 0
# Occasion residual variances all constrained equal
  outcome.0 ~~ (ResVar)*outcome.0; outcome.1 ~~ (ResVar)*outcome.1 
  outcome.2 ~~ (ResVar)*outcome.2; outcome.3 ~~ (ResVar)*outcome.3
"
ModelEmptyRI = lavaan(model=SyntaxEmptyRI, data=Example5_wide, estimator="ML", mimic="mplus")
summary(ModelEmptyRI, fit.measures=TRUE, standardized=FALSE)
print("Model-implied marginal means, variances, and covariances; add correlations=ICC")
fitted(object=ModelEmptyRI); lavInspect(object=ModelEmptyRI, "cor.ov")

txtComment(" ") # insert blank space in output


print("R lavaan Eq 5.3: Fixed Linear Time, Random Intercept Model")
SyntaxFixLinRI = "
# Define intercept factor by fixing all factor loadings to 1
  FactInt =~ 1*outcome.0 + 1*outcome.1 + 1*outcome.2 + 1*outcome.3
# Define linear time slope factor by fixing loadings to time values
  FactLin =~ 0*outcome.0 + 1*outcome.1 + 2*outcome.2 + 3*outcome.3
# Estimate factor means = fixed intercept and fixed linear time slope
  FactInt ~ (FixInt)*1; FactLin ~ (FixLin)*1
# Estimate intercept factor variance = random intercept variance
  FactInt ~~ (IntVar)*FactInt
# Fix linear time slope factor variance to 0 (not random yet)
  FactLin ~~ 0*FactLin
# Occasion intercepts all fixed to 0
  outcome.0 ~ 0; outcome.1 ~ 0; outcome.2  ~ 0; outcome.3 ~ 0
# Occasion residual variances all constrained equal
  outcome.0 ~~ (ResVar)*outcome.0; outcome.1 ~~ (ResVar)*outcome.1 
  outcome.2 ~~ (ResVar)*outcome.2; outcome.3 ~~ (ResVar)*outcome.3
"
ModelFixLinRI = lavaan(model=SyntaxFixLinRI, data=Example5_wide, estimator="ML", mimic="mplus")
summary(ModelFixLinRI, fit.measures=TRUE, standardized=FALSE)
print("Model-implied marginal means, variances, and covariances; add correlations")
fitted(object=ModelFixLinRI); lavInspect(object=ModelFixLinRI, "cor.ov")

txtComment(" ") # insert blank space in output


print("R lavaan Eq 5.5: Random Linear Time Model")
SyntaxRandLin = "
# Define intercept factor by fixing all factor loadings to 1
  FactInt =~ 1*outcome.0 + 1*outcome.1 + 1*outcome.2 + 1*outcome.3
# Define linear time slope factor by fixing loadings to time values
  FactLin =~ 0*outcome.0 + 1*outcome.1 + 2*outcome.2 + 3*outcome.3
# Estimate factor means = fixed intercept and fixed linear time slope
  FactInt ~ (FixInt)*1; FactLin ~ (FixLin)*1
# Estimate intercept factor variance = random intercept variance
  FactInt ~~ (IntVar)*FactInt
# Estimate linear time slope factor variance = random slope variance
  FactLin ~~ (LinVar)*FactLin
# Estimate random effects covariance
  FactInt ~~ (ILcov)*FactLin
# Occasion intercepts all fixed to 0
  outcome.0 ~ 0; outcome.1 ~ 0; outcome.2  ~ 0; outcome.3 ~ 0
# Occasion residual variances all constrained equal
  outcome.0 ~~ (ResVar)*outcome.0; outcome.1 ~~ (ResVar)*outcome.1 
  outcome.2 ~~ (ResVar)*outcome.2; outcome.3 ~~ (ResVar)*outcome.3
"
ModelRandLin = lavaan(model=SyntaxRandLin, data=Example5_wide, estimator="ML", mimic="mplus")
summary(ModelRandLin, fit.measures=TRUE, standardized=FALSE)
print("Model-implied marginal means, variances, and covariances; add correlations")
fitted(object=ModelRandLin); lavInspect(object=ModelRandLin, "cor.ov")
print("Does random linear time slope improve fit?")
anova(ModelRandLin, ModelFixLinRI) # Remove random slope and covariance

txtComment(" ") # insert blank space in output

#### NOTE: THE FOLLOWING DOES NOT WORK, BUT IS MY BEST ATTEMPT #####
#print("R lavaan Ch 5: Random Linear Time Model + AR1 R Matrix")
#SyntaxRandLinAR1 = "
# Define intercept factor by fixing all factor loadings to 1
#  FactInt =~ 1*outcome.0 + 1*outcome.1 + 1*outcome.2 + 1*outcome.3
# Define linear time slope factor by fixing factor loadings to time value
#  FactLin =~ 0*outcome.0 + 1*outcome.1 + 2*outcome.2 + 3*outcome.3
# Estimate factor means = fixed intercept and fixed linear time slope
#  FactInt ~ (FixInt)*1; FactLin ~ (FixLin)*1
# Estimate intercept factor variance = random intercept variance
#  FactInt ~~ (IntVar)*FactInt
# Estimate linear time slope factor variance = random slope variance
#  FactLin ~~ (LinVar)*FactLin
# Estimate random effects covariance
#  FactInt ~~ (ILcov)*FactLin
# Occasion intercepts all fixed to 0
#  outcome.0 ~ 0; outcome.1 ~ 0; outcome.2  ~ 0; outcome.3 ~ 0
# Occasion residual variances all constrained equal
#  outcome.0 ~~ (ResVar)*outcome.0; outcome.1 ~~ (ResVar)*outcome.1 
#  outcome.2 ~~ (ResVar)*outcome.2; outcome.3 ~~ (ResVar)*outcome.3
# Lag-1 covariances all constrained equal
#  outcome.0 ~~ (Rcov1)*outcome.1; outcome.1 ~~ (Rcov1)*outcome.2; outcome.2 ~~ (Rcov1)*outcome.3
# Lag-2 covariances all constrained equal
#  outcome.0 ~~ (Rcov2)*outcome.2; outcome.1 ~~ (Rcov2)*outcome.3
# Lag-3 covariance
#  outcome.0 ~~ (Rcov3)*outcome.3
# Lagged correlations made using covariances
#  Rcor1 := Rcov1/(sqrt(ResVar)*sqrt(ResVar))
#  Rcor2 := Rcov2/(sqrt(ResVar)*sqrt(ResVar))
#  Rcor3 := Rcov3/(sqrt(ResVar)*sqrt(ResVar))
# Constrain correlations to have AR1 pattern
#  Rcor2 == Rcor1*Rcor1
#  Rcor3 == Rcor1*Rcor1*Rcor1
#"
#ModelRandLinAR1 = lavaan(model=SyntaxRandLinAR1, data=Example5_wide, estimator="ML", mimic="mplus")
#summary(ModelRandLinAR1, fit.measures=TRUE, standardized=FALSE)
#print("Model-implied marginal means, variances, and covariances; add correlations")
#fitted(object=ModelRandLinAR1); lavInspect(object=ModelRandLinAR1, "cor.ov")
#print("Does random linear time slope improve fit?")
#anova(ModelRandLinAR1, ModelRandLin) # Remove AR1 correlation

txtComment(" ") # insert blank space in output


print("R lavaan Ch 5: RRandom Linear Time in G + Toeplitz Lag-1 Residual Covariance in R")
SyntaxRandLinTP2 = "
# Define intercept factor by fixing all factor loadings to 1
  FactInt =~ 1*outcome.0 + 1*outcome.1 + 1*outcome.2 + 1*outcome.3
# Define linear time slope factor by fixing factor loadings to time value
  FactLin =~ 0*outcome.0 + 1*outcome.1 + 2*outcome.2 + 3*outcome.3
# Estimate factor means = fixed intercept and fixed linear time slope
  FactInt ~ (FixInt)*1; FactLin ~ (FixLin)*1
# Estimate intercept factor variance = random intercept variance
  FactInt ~~ (IntVar)*FactInt
# Estimate linear time slope factor variance = random slope variance
  FactLin ~~ (LinVar)*FactLin
# Estimate random effects covariance
  FactInt ~~ (ILcov)*FactLin
# Occasion intercepts all fixed to 0
  outcome.0 ~ 0; outcome.1 ~ 0; outcome.2  ~ 0; outcome.3 ~ 0
# Occasion residual variances all constrained equal
  outcome.0 ~~ (ResVar)*outcome.0; outcome.1 ~~ (ResVar)*outcome.1 
  outcome.2 ~~ (ResVar)*outcome.2; outcome.3 ~~ (ResVar)*outcome.3
# Lag-1 covariances all constrained equal
  outcome.0 ~~ (Rcov1)*outcome.1; outcome.1 ~~ (Rcov1)*outcome.2; outcome.2 ~~ (Rcov1)*outcome.3
"
ModelRandLinTP2 = lavaan(model=SyntaxRandLinTP2, data=Example5_wide, estimator="ML", mimic="mplus")
summary(ModelRandLinTP2, fit.measures=TRUE, standardized=FALSE)
print("Model-implied marginal means, variances, and covariances; add correlations")
fitted(object=ModelRandLinTP2); lavInspect(object=ModelRandLinTP2, "cor.ov")
print("Does random linear time slope improve fit?")
anova(ModelRandLinTP2, ModelRandLin) # Test TOEP lag-1 covariance

# Close output text file
txtStop()




